#------------------------------------------------------------------
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/CiscoUCS/CiscoUCS.pm,v 1.3 2014/09/18 18:16:31 kkorde Exp $
#
# $sAuthor: $
#
# $Date: 2014/09/18 18:16:31 $
# $Revision: 1.3 $
#
# Provides:                            CiscoUCS
# Author:                              kkorde
# crAgent Build:                       \$Id: crAgent,v 1.100 201
# crAgent Run Date:                    01 Aug, 2013
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

use strict;

package CiscoUCS;

use ag_i18n_inc;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#--------------------------------
# Include standard PERL modules..
#--------------------------------
use Fcntl;
use Socket;
use File::Path qw ( mkpath );
use File::Basename;
use File::stat;

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: CiscoUCS.pm,v 1.3 2014/09/18 18:16:31 kkorde Exp $::ETVHeader";
Version::TellVersion ( $sVer );

my $AGENT_HOME= "/opt/VRTSagents/ha";
my $VCSLOG = $ENV{"VCS_LOG"} || "/var/VRTSvcs";

#--------------------------------
# Define CiscoUCS subroutines..
#--------------------------------

sub ArgsValid {

  my ( $sEntryPointName, $localUCSConfigFile ) = @_;

  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  UCSConfigFile            [%s]",
  $sMyName, $sEntryPointName, $localUCSConfigFile );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName UCSConfigFile );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my @lsOnlineArgs  = qw ( UCSConfigFile );
  my @lsOfflineArgs = qw ( UCSConfigFile );
  my @lsMonitorArgs = qw ( UCSConfigFile );
  my @lsCleanArgs   = qw ( UCSConfigFile );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs );

  my @lsReqdArgs = @{ $hlsNeeded { $sEntryPointName } };

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  if ( $sEntryPointName eq "online"  ||
       $sEntryPointName eq "offline" || 
       $sEntryPointName eq "clean" ){

      if ( ! -e $localUCSConfigFile) {
          &logError ( 20003,
          "<%s> UCS Configuration file [%s] does not exist", $sMyName, $localUCSConfigFile);
          $iReturnCode = 0; # Failure..
     }
  }

  $iReturnCode ||
  &logWarn ( 20004,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

sub Start {

  my ( $sUCSConfigFile ) = @_;

  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20005, 
  "Subroutine <%s> called with:  UCSConfigFile [%s]", $sMyName, $sUCSConfigFile );

  my $iReturnCode = 1;
  my ( $sCommand, $sCommandArgs, $iTillWhen, $iExitCode, $iTimeout, @lsCmdout ) = ();

  $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );
 
  $sCommand = "$AGENT_HOME/bin/CiscoUCS/ucsha";
  $sCommandArgs = "-start $sUCSConfigFile";

  ( @lsCmdout, $iExitCode, $iTimeout ) = ();

  # Request startup..
  #------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20006,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );
    $iReturnCode = 0; # Failure..
  } elsif ( 1 ==
    CiscoUCS::ExecLocalCmd ( "root",
                             "",
                             $sCommand,
                             $sCommandArgs,
                             \$iExitCode,
                             $iTimeout,
                             \@lsCmdout ) ) {
    &logTrace ( 20007,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
    $iReturnCode = 1;
  } else {
    &logError ( 20008,
    "<%s> Cmd [%s] with args [%s] execution failed", $sMyName, $sCommand, $sCommandArgs );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}

sub Stop {

  my ( $sUCSConfigFile ) = @_;

  my $sMyName = ( caller 0 )[3];
  my $iReturnCode = 1;

  &logTrace ( 20009, "Subroutine <%s> called with:  UCSConfigFile            [%s]", $sMyName, $sUCSConfigFile );

  my ( $sCommand, $sCommandArgs, $iTillWhen, $iExitCode, $iTimeout, @lsCmdout ) = ();

  $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );

  $sCommand = "$AGENT_HOME/bin/CiscoUCS/ucsha";
  $sCommandArgs = "-stop -agent $sUCSConfigFile";

  ( @lsCmdout, $iExitCode, $iTimeout ) = ();

  # Request startup..
  #------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20010,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  } elsif ( 1 == CiscoUCS::ExecLocalCmd ( "root",
                             "",
                             $sCommand,
                             $sCommandArgs,
                             \$iExitCode,
                             $iTimeout,
                             \@lsCmdout ) ) {
    &logTrace ( 20011,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
    $iReturnCode = 1;
  } else {
    &logError ( 20012,
    "<%s> Cmd [%s] with args [%s] execution failed", $sMyName, $sCommand, $sCommandArgs );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}

sub GetUniqueID {

    my ($AdminURL) = (@_);

    my $sMyName = ( caller 0 )[3];
    &logTrace ( 20013, "Subroutine <%s> called with:
    AdminURL [%s]", $sMyName, $AdminURL);

    my ($UniqueID) = ();
    if($AdminURL =~ /\S+:\/\/(.*)\//){
       $UniqueID = $1;
       $UniqueID =~ s/\./_/g;
    } else {
       #TBD
    }

    return $UniqueID;
}

sub GetUCSConfigData{

    my ($UCSConfigFile) = @_;
    my $sMyName = ( caller 0 )[3];
    my (%UCSData) = ();
    my @lsReqKeys = qw /UCS_MANAGER_URL UCS_ADMIN UCS_PASSWORD/;

    &logTrace (20014, "Subroutine <%s> called with:
    UCSConfigFile [%s]", $sMyName, $UCSConfigFile);

    if(! -f $UCSConfigFile){
       &logTrace ( 20015, "<%s> UCSConfigFile [%s] does not exist", $sMyName, $UCSConfigFile);
       return (1, \%UCSData);
    }

    if(open(MYFILE, $UCSConfigFile)){
       while(<MYFILE>){
          if ( $_ =~ /(.*)=\[(.*)\]/ ){
               $UCSData{$1} = $2;
          }
       }
       close(MYFILE);
       foreach my $key(@lsReqKeys){
          if(! exists $UCSData{$key}){
             &logTrace (20016, "<%s> Required parameter [%s] not present in [%s].", $sMyName, $key, $UCSConfigFile);
             return (1, \%UCSData);
          }
       }
    } else {
       &logTrace ( 20017, "<%s> Unable to open UCSConfigFile [%s]", $sMyName, $UCSConfigFile);
       return (1,\%UCSData);
    }
    
    return (0,\%UCSData);
}


sub GetSPState {

  my ( $UCSConfigFile, $EntryPointName ) = @_;

  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20018, "Subroutine <%s> called with:
  UCSConfigFile [%s]", $sMyName, $UCSConfigFile);

  my $iReturnCode =  VCSDef_MonitorExit::ONLINE;

  my ( $UniqueID, $hrefUCSData, $sUCSAdminURL, $UCSSPDataFile) = ();
  my ( $iTillWhen, $iTimeout, $iExitCode, $sCommand, $sCommandArgs, @lsCmdout, $retVal ) = ();
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };

  if(($EntryPointName eq "monitor") && ( ! -e $UCSConfigFile )){
      $iReturnCode =  VCSDef_MonitorExit::OFFLINE;
      return $iReturnCode;
  }

  ($retVal, $hrefUCSData) = GetUCSConfigData($UCSConfigFile);

  if($retVal == 1){
     $iReturnCode =  VCSDef_MonitorExit::UNKNOWN;
     &logError ( 20019, "<%s> Unable to fetch data from UCSConfigFile [%s]", $sMyName, $UCSConfigFile);
     return $iReturnCode;
  }

  $sUCSAdminURL  = ${$hrefUCSData}{UCS_MANAGER_URL};
  $UniqueID      = GetUniqueID($sUCSAdminURL);
  $UCSSPDataFile = "$VCSLOG/log/tmp/CiscoUCS/${UniqueID}_UCSServiceProfileData";

  # Below is the intentional offline case when the user exectues "./ucsha -stop" to
  # stop the Service Profile Monitoring

  if( (-e "$VCSLOG/log/tmp/CiscoUCS/${UniqueID}_INTENTIONAL") && (! -e $UCSSPDataFile) ){

     my $FileIntentional = "$VCSLOG/log/tmp/CiscoUCS/${UniqueID}_INTENTIONAL";
     &logTrace ( 20020, "<%s> File [%s] indicates utility ucsha with stop option was executed externally.", $sMyName, $FileIntentional);
     system("/bin/rm -rf $VCSLOG/log/tmp/CiscoUCS/${UniqueID}_INTENTIONAL");
     &logTrace ( 20021, "<%s> Returning Intentional Offline.", $sMyName);
     return 200;

  } elsif(! -e $UCSSPDataFile){

      &logTrace ( 20022, "<%s> Service Profile Data File[%s] does not exist", $sMyName, $UCSSPDataFile);
      $iReturnCode = VCSDef_MonitorExit::OFFLINE;
      return $iReturnCode;

  }else{
      #---------------------------------------------------------------------
      # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
      #---------------------------------------------------------------------
      $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );

      $sCommand     = "$AGENT_HOME/bin/CiscoUCS/ucsha";
      $sCommandArgs = "-monitor $UCSConfigFile";

      if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
             &logError ( 20023, "<%s> Insufficient time to proceed with ucsha monitor. Short by [%s] sec",
                         $sMyName, abs ( $iTimeout ) );
           $iReturnCode = 0; # Failure..
      }elsif( 1 ==  CiscoUCS::ExecLocalCmd ( "root",
                                             "",
                                             $sCommand,
                                             $sCommandArgs,
                                             \$iExitCode,
                                             $iTimeout,
                                             \@lsCmdout )){
          &logTrace (20024,"<%s> Cmd [%s] with args [%s] executed successfully",$sMyName, $sCommand, $sCommandArgs );
          $iReturnCode = VCSDef_MonitorExit::ONLINE;
      } else {
          &logError ( 20025,
          "<%s> Cmd [%s] with args [%s] execution failed", $sMyName, $sCommand, $sCommandArgs );
          $iReturnCode = 0;
      }
      return $iReturnCode;
  }
}

=pod
 -------------------------------------------------------------------------
 Function : bool ExecLocalCmd ( IN String User,
                                IN String ,
                                IN String Command,
                                IN String CommandArgs,
                                OUT RefInt ExitCode,
                                IN Integer Timeout,
                                OUT ListReference Cmdout );

 Description : Wrapper function for
               Sys::RunWithEnvCmdWithOutputWithTimeOut to collect output
               of the command.
 
 Parameters :

 $sUser            UNIX user to run the command as
 $sEnvFile         Absolute path to location of EnvFile to source
 $sCommand         Absolute path to location of command
 $sCommandArgs     Arguments to command
 $riExitCode       Reference to collect exit code of command
 $iTimeout         Time limit to execute the command
 $rlsCmdout        Stores o/p from command if successful
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLocalCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sEnvFile,
       $sCommand,
       $sCommandArgs,
       $riExitCode,
       $iTimeout,
       $rlsCmdout ,
       $sOutputFilename ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20026, "Subroutine <%s> called with:
  User        [%s]
  EnvFile     [%s]
  Command     [%s]
  CommandArgs [%s]
  Timeout     [%s]",
  $sMyName,
  $sUser, $sEnvFile, $sCommand, $sCommandArgs, $iTimeout);

  my $iReturnCode = 1;

  #-------------------
  # Set up arguments..
  #-------------------
  my $bSourceUserEnv = 0; # Do not source..
  my $bTimedOut = 0;      # Timeout variable..

  @$rlsCmdout = (); # Reset..

  #-----------
  # Validate..
  #-----------
  if ( ! -e $sCommand ) {
    &logError ( 20027,
    "<%s> Cmd [%s] does not exist", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( ! -x $sCommand ) {
    &logError ( 20028,
    "<%s> Cmd [%s] is not executable", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( $iTimeout <= 1 ) {
    &logError ( 20029,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with Args [%s].",
    $sMyName, $iTimeout, $sCommand, $sCommandArgs );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return if failed.. 

  #----------------------
  # Set LOGNAME to user..
  #----------------------
  my $sOldLogName = $ENV { LOGNAME }; # Backup..
  $ENV { LOGNAME } = $sUser;          # Now set..

  &logTrace ( 20030,
  "<%s> Executing Command [%s] with Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut ( $sEnvFile,
                                                           $bSourceUserEnv,
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           $riExitCode,
                                                           $sUser,
                                                           $rlsCmdout,
                                                           \$bTimedOut,
                                                           $iTimeout);

  #---------------------------
  # Remove trailing newlines..
  #---------------------------
  chomp @$rlsCmdout;

  #--------------------
  # Check return code..
  #--------------------
  if ( $iReturnCode ) {   # No error with command execution..
    if ( $$riExitCode ) { # Failure..
      &logTrace ( 20031,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
      $iReturnCode = 0; # Failure..
    }
    else {
      &logTrace ( 20032,
      "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
    }
  }
  else { # Execution error..
    @$rlsCmdout = (); # Reset..

    if ( $bTimedOut ) {  # We timed out..
      &logError ( 20033,
      "<%s> Command [%s] with args [%s] did not complete within [%s] seconds",
      $sMyName, $sCommand, $sCommandArgs, $iTimeout );
      $$rlsCmdout [0] = 'timeout'; # Timeout..
    }
    else { # Failure due to something else..
      &logError ( 20034,
      "<%s> Failed in call to RunWithEnvCmdWithOutputWithTimeOut",
      $sMyName );
    }
  }

  #----------
  # Restore..
  #----------
  defined $sOldLogName ? $ENV { LOGNAME } = $sOldLogName : delete $ENV { LOGNAME };

  return $iReturnCode;
}

1; # So PERL knows its included..
